@@ -386,6 +386,8 @@ class GraphRecommender:
         embeddings node x embedding_dim tensor"""
 
         if self._embeddings is None:
+            if not self._masks_set:
+                self.set_masks()
             print('computing embeddings for all nodes...')
             with th.no_grad():
                 self._embeddings = self.net.inference(
@@ -401,10 +403,9 @@ class GraphRecommender:
         -------
         a faiss index with input embeddings added and optionally trained"""
 
-        if not self._masks_set:
-            self.set_masks()
-
         if self._index is None:
+            if not self._masks_set:
+                self.set_masks()
             if self.distance_metric=='cosine':
                 self._index  = faiss.IndexFlatIP(self.embedding_dim)
                 embeddings = np.copy(self.embeddings[self.entity_mask])
@@ -451,16 +452,18 @@ class GraphRecommender:
         D, I = self.index.search(inputs,k)
         return D,I
 
-    def query_neighbors(self, nodelist, k):
+    def query_neighbors(self, nodelist, k, return_labels=True):
         """For each query node in nodelist, return the k closest neighbors in the 
         embedding space.
 
         Args
         ----
-        nodelist : list of any types castable to stirng
+        nodelist : list
             list of node identifiers to query
         k : int
             number of neighbors to return
+        return_labels : bool
+            if true, includes the node label of all neighbors returned
 
         Returns
         -------
@@ -469,18 +472,25 @@ class GraphRecommender:
         if not self._masks_set:
             self.set_masks()
 
+        relevant_nodes = self.node_ids.loc[self.node_ids.id.isin(nodelist)]
         try:
-            intids = [self.node_ids.loc[self.node_ids.id == node].intID.iloc[0]
+            intids = [relevant_nodes.loc[relevant_nodes.id == node].intID.iloc[0]
                         for node in nodelist]
         except IndexError:
-            intids = [self.node_ids.loc[self.node_ids.id == int(node)].intID.iloc[0]
+            intids = [relevant_nodes.loc[relevant_nodes.id == int(node)].intID.iloc[0]
                         for node in nodelist]
 
         inputs = self.embeddings[intids,:]
         D, I = self._search_index(inputs,k)
-        faissid_to_nodeid = self.node_ids.id.to_numpy()[self.entity_mask]
-        I = [[faissid_to_nodeid[neighbor] for neighbor in neighbors] for neighbors in I]
-        output = {node:{'neighbors':i,'distances':d.tolist()} for node, d, i in zip(nodelist,D,I)}
+        faissid_to_nodeid = self.node_ids.id.to_numpy()[self.entity_mask].tolist()
+        if return_labels:
+            faissid_to_label = self.node_ids.classid.to_numpy()[self.entity_mask].tolist()
+            L = [[faissid_to_label[neighbor] for neighbor in neighbors] for neighbors in I]
+            I = [[faissid_to_nodeid[neighbor] for neighbor in neighbors] for neighbors in I]
+            output = {node:{'neighbors':i,'neighbor labels':l,'distances':d.tolist()} for node, d, i, l in zip(nodelist,D,I,L)}
+        else:
+            I = [[faissid_to_nodeid[neighbor] for neighbor in neighbors] for neighbors in I]
+            output = {node:{'neighbors':i,'distances':d.tolist()} for node, d, i in zip(nodelist,D,I)}
         return output
 
     def evaluate(self, test_levels=[5,1], test_only=False):
@@ -775,7 +785,7 @@ class GraphRecommender:
         pathlib.Path(production_path).mkdir(exist_ok=True)
         self.save(production_path)
         os.environ['FASTREC_DEPLOY_PATH'] = production_path
-        #this import cant be at the top level to prevent circular depedency
+        #this import cant be at the top level to prevent circular dependency
         from RecAPI import app
         uvicorn.run(app,*args,**kwargs)
 
@@ -790,19 +800,42 @@ class GraphRecommender:
         filepath : str 
             path on disk to save files"""
 
-        with open(f'{filepath}/dgl.pkl','wb') as pklf:
-            pickle.dump(self.G,pklf)
+
+        outg = dgl.as_immutable_graph(self.G)
+        dgl.data.utils.save_graphs(f'{filepath}/dgl.bin',outg)
 
         self.node_ids.to_csv(f'{filepath}/node_ids.csv',index=False)
 
-        with open(f'{filepath}/embed.pkl','wb') as pklf:
-            pickle.dump(self.embed,pklf)
-
+        th.save(self.embed,f'{filepath}/embed.torch')
         th.save(self.net.state_dict(),f'{filepath}/model_weights.torch')
+        embeddings = self.embeddings
+        np.save(f'{filepath}/final_embed.npy',embeddings,allow_pickle=False)
 
         with open(f'{filepath}/initargs.pkl','wb') as pklf:
             pickle.dump(self.initargs,pklf)
 
+    def load_graph_data(self,filepath):
+        """Restore graph data from disk, but not network parameters
+        or trained embeddings. Useful for changing network parameters
+        if you don't want to reconstruct the graph.
+
+        Args
+        ----
+        filepath : str
+            path to where you saved previous the GraphRecommender
+        """
+
+        self.G,_ = dgl.data.utils.load_graphs(f'{filepath}/dgl.bin')
+        self.G = restored_self.G[0]
+        self.G.readonly()
+        self.G = dgl.as_heterograph(restored_self.G)
+
+        self.node_ids = pd.read_csv(f'{filepath}/node_ids.csv')
+
+        self._masks_set = False
+        self._embeddings = None 
+        self._index = None 
+
 
     @classmethod
     def load(cls, filepath, device=None, faiss_gpu=None):
@@ -850,16 +883,17 @@ class GraphRecommender:
                             p_train,
                             train_faiss_index)
 
-        with open(f'{filepath}/dgl.pkl','rb') as pklf:
-            restored_self.G = pickle.load(pklf)
+        restored_self.G,_ = dgl.data.utils.load_graphs(f'{filepath}/dgl.bin')
+        restored_self.G = restored_self.G[0]
+        restored_self.G.readonly()
+        restored_self.G = dgl.as_heterograph(restored_self.G)
 
         restored_self.node_ids = pd.read_csv(f'{filepath}/node_ids.csv')
-        restored_self.node_ids.id = restored_self.node_ids.id.astype(str) 
-
-        with open(f'{filepath}/embed.pkl','rb') as pklf:
-            restored_self.embed = pickle.load(pklf)
 
+        restored_self.embed = th.load(f'{filepath}/embed.torch',map_location=th.device(torch_device))
         restored_self.net.load_state_dict(th.load(f'{filepath}/model_weights.torch',map_location=th.device(torch_device)))
+        embeddings = np.load(f'{filepath}/final_embed.npy',allow_pickle=False)
+        restored_self._embeddings = embeddings
 
         return restored_self
 
